from enum import IntEnum
import serial
import struct
import time

# https://gitee.com/alicedodo/xaobao_cheap_bus_servo_hack_record

class Servo:
	# With header 0xFAAF
	class CommandID(IntEnum):
		Set_Angle = 0x01
		Get_Angle = 0x02
		LED = 0x04
		Set_ID = 0xCD
		Set_Angle_Correction = 0xD2
		Get_Angle_Correction = 0xD4

	# With header 0xFCCF
	class FirmwareCommandID(IntEnum):
		Get_Version = 0x01
		Upgrade_Firmware = 0x02

	def __init__(self):
		# Header Data
		self.headerH = 0xFA
		self.headerL = 0xAF
		self.id = 0
		self.commandID = 0
		# Data
		self.param1H = 0
		self.param1L = 0
		self.param2H = 0
		self.param2L = 0
		# Checksum
		self.checksum = 0
		# End
		self.endCode = 0xED

	def Encode(self):
		# Pack
		buffer = bytearray(10)
		buffer[0] = self.headerH
		buffer[1] = self.headerL
		buffer[2] = self.id
		buffer[3] = self.commandID
		buffer[4] = self.param1H
		buffer[5] = self.param1L
		buffer[6] = self.param2H
		buffer[7] = self.param2L
		# Calculate checksum
		self.checksum = 0
		for b in range(6):
			self.checksum = (self.checksum + buffer[2 + b]) & 0xFF
		# Add checksum and end code
		buffer[8] = self.checksum
		buffer[9] = self.endCode
		# Return buffer
		return buffer
	
	def SetID(self, id):
		# Set Command ID
		self.commandID =  Servo.CommandID.Set_ID
		self.id = id
		# Set Parameters
		self.param1H = 0	# 0 padding
		self.param1L = 0	# 0 padding
		self.param2H = 0	# 0 padding
		self.param2L = 0	# 0 padding
		# Encode
		return self.Encode()

	def SetAngle(self, id, angle, speed, lockTime):
		# Check for limits
		if angle > 240:
			angle = 240
		elif angle < 0:
			angle = 0
		if speed > 255:
			speed = 255
		elif speed < 0:
			speed = 0
		if lockTime > 3270:
			lockTime = 3270
		elif lockTime < 0:
			lockTime = 0
		# Set Command ID
		self.commandID = Servo.CommandID.Set_Angle
		self.id = id
		# Set Parameters
		self.param1H = angle	# 0-240
		self.param1L = speed	# 0-255 in 20ms steps
		self.param2H = (int)(lockTime >> 8)		# 0-3270 in 20ms steps
		self.param2L = (int)(lockTime & 0xFF)
		# Encode
		return self.Encode()

port = "COM3"
baud = 115200

try:
	servoSerial = serial.Serial(port = port, 
			     				baudrate = baud, 
								bytesize = serial.EIGHTBITS, 
								stopbits = serial.STOPBITS_ONE, 
								parity = serial.PARITY_NONE)
except serial.SerialException as e:
	print("Could not open serial port '{}': {}".format(port, e))

servo = Servo()
# Speed limit is:
# Vin (min) = 6.1V: 150*20ms for 180 Degrees
# Vin (typ) = 7.6V: 0*20ms for 180 Degrees (no limitations)
# Vin (max) = 8.5V: 0*20ms for 180 Degrees (no limitations)
buf = servo.SetAngle(0, 180, 0, 1000)
print(buf.hex())
servoSerial.write(buf)
